# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import six

from six import BytesIO
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, Multiple, String


class EmpowerBase(Splitted):

    SEPARATOR = "\t"


class EmpowerResult(object):
    FIELDS = [
        String("sample_name"),
        Multiple("results", (String("resu"),), separator="\t"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class EmpowerResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: EmpowerResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str("\t"))
        for row in reader:
            if six.PY2:
                line = [c.decode("utf-8") for c in row]
            else:
                line = row
            if not len(line):
                continue
            if len(line) >= 2:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "empower", "label": "Empower WATERS"}

    def parse(self, data):
        obj = EmpowerResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.sample_name == "SampleName":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(sample_id=m.sample_name, type="patient")
                for index, result in enumerate(m.results):
                    if RESULTS_CODES[index]:
                        sample.results.add(
                            code=RESULTS_CODES[index].strip(), value=result.strip()
                        )
        return worklist
